import base64
import os
import gzip
from starlette.applications import Starlette
from starlette.responses import Response
from starlette.routing import Route

random_5k = base64.b64encode(os.urandom(5 * 1024)).decode('utf-8')
random_5k = gzip.compress(random_5k.encode('utf-8'))

random_50k = base64.b64encode(os.urandom(50 * 1024)).decode('utf-8')
random_50k = gzip.compress(random_50k.encode('utf-8'))


def gzip_response(gzipped_content):
    headers = {
        'Content-Encoding': 'gzip',
        'Content-Length': str(len(gzipped_content)),
    }
    return Response(gzipped_content, headers=headers)

app = Starlette(
    routes=[
        Route("/5k", lambda r: gzip_response(random_5k)),
        Route("/50k", lambda r: gzip_response(random_50k)),
    ],
)

# Run server: uvicorn server:app



import time
from concurrent.futures import ThreadPoolExecutor, as_completed
from importlib.metadata import version
import pandas as pd
import requests
import httpx
import tls_client
import pyreqwest_impersonate
import curl_cffi.requests

results = []
PACKAGES = [
    ("requests", requests.Session),
    ("httpx", httpx.Client),
    ("tls_client", tls_client.Session),
    ("curl_cffi", curl_cffi.requests.Session),
    ("pyreqwest_impersonate", pyreqwest_impersonate.Client),
]


def add_package_version(packages):
    return [(f"{name} {version(name)}", classname) for name, classname in packages]


def session_get_test(session_class, requests_number):
    s = session_class()
    for _ in range(requests_number):
        s.get(url).text


PACKAGES = add_package_version(PACKAGES)

# one thread
requests_number = 2000
for response_size in ["5k", "50k"]:
    url = f"http://127.0.0.1:8000/{response_size}"
    print(f"\nOne worker, {response_size=}, {requests_number=}")
    for name, session_class in PACKAGES:
        start = time.perf_counter()
        cpu_start = time.process_time()
        session_get_test(session_class, requests_number)
        dur = round(time.perf_counter() - start, 3)
        cpu_dur = round(time.process_time() - cpu_start, 3)
        results.append(
            {
                "name": name,
                "threads": 1,
                "size": response_size,
                "time": dur,
                "cpu_time": cpu_dur,
            }
        )
        print(f"    name: {name:<30} time: {dur} cpu_time: {cpu_dur}")


# multiple threads
requests_number = 500
threads_number = 4
for response_size in ["5k", "50k"]:
    url = f"http://127.0.0.1:8000/{response_size}"
    print(f"\n{threads_number} workers, {response_size=}, {requests_number=}")
    for name, session_class in PACKAGES:
        start = time.perf_counter()
        cpu_start = time.process_time()
        with ThreadPoolExecutor(threads_number) as executor:
            futures = [
                executor.submit(session_get_test, session_class, requests_number)
                for _ in range(threads_number)
            ]
            for f in as_completed(futures):
                f.result()
        dur = round(time.perf_counter() - start, 3)
        cpu_dur = round(time.process_time() - cpu_start, 3)
        results.append(
            {
                "name": name,
                "threads": threads_number,
                "size": response_size,
                "time": dur,
                "cpu_time": cpu_dur,
            }
        )
        print(f"    name: {name:<30} time: {dur} cpu_time: {cpu_dur}")


df = pd.DataFrame(results)
pivot_df = df.pivot_table(
    index=["name", "threads"],
    columns="size",
    values=["time", "cpu_time"],
    aggfunc="mean",
)
pivot_df.reset_index(inplace=True)
pivot_df.columns = [" ".join(col).strip() for col in pivot_df.columns.values]
pivot_df = pivot_df[
    ["name", "threads"]
    + [col for col in pivot_df.columns if col not in ["name", "threads"]]
]
unique_threads = pivot_df["threads"].unique()
for thread in unique_threads:
    thread_df = pivot_df[pivot_df["threads"] == thread]
    print(f"\nTable for {thread} threads:")
    print(thread_df.to_string(index=False))
    thread_df.to_csv(f"{thread}_threads.csv", index=False)



from time import sleep
from urllib.parse import parse_qs

from pyreqwest_impersonate import Client


def retry(max_retries=3, delay=1):
    def decorator(func):
        def wrapper(*args, **kwargs):
            for attempt in range(max_retries):
                try:
                    return func(*args, **kwargs)
                except Exception as e:
                    if attempt < max_retries - 1:
                        sleep(delay)
                        continue
                    else:
                        raise e

        return wrapper

    return decorator

@retry()
def test_client_init_params():
    auth = ("user", "password")
    headers = {"X-Test": "test"}
    params = {"x": "aaa", "y": "bbb"}
    client = Client(auth=auth, params=params, headers=headers, verify=False)
    response = client.get("https://httpbin.org/anything")
    assert response.status_code == 200
    json_data = response.json()
    assert json_data["headers"]["X-Test"] == "test"
    assert json_data["headers"]["Authorization"] == "Basic dXNlcjpwYXNzd29yZA=="
    assert json_data["args"] == {"x": "aaa", "y": "bbb"}


@retry()
def test_client_get():
    client = Client(verify=False)
    auth_bearer = "bearerXXXXXXXXXXXXXXXXXXXX"
    headers = {"X-Test": "test"}
    params = {"x": "aaa", "y": "bbb"}
    response = client.get(
        "https://httpbin.org/anything",
        auth_bearer=auth_bearer,
        headers=headers,
        params=params,
    )
    assert response.status_code == 200
    json_data = response.json()
    assert json_data["headers"]["X-Test"] == "test"
    assert json_data["headers"]["Authorization"] == "Bearer bearerXXXXXXXXXXXXXXXXXXXX"
    assert json_data["args"] == {"x": "aaa", "y": "bbb"}
    assert "Bearer bearerXXXXXXXXXXXXXXXXXXXX" in response.text
    assert b"Bearer bearerXXXXXXXXXXXXXXXXXXXX" in response.content


@retry()
def test_client_post_content():
    client = Client(verify=False)
    auth = ("user", "password")
    headers = {"X-Test": "test"}
    params = {"x": "aaa", "y": "bbb"}
    content = b"test content"
    response = client.post(
        "https://httpbin.org/anything",
        auth=auth,
        headers=headers,
        params=params,
        content=content,
    )
    assert response.status_code == 200
    json_data = response.json()
    assert json_data["headers"]["X-Test"] == "test"
    assert json_data["headers"]["Authorization"] == "Basic dXNlcjpwYXNzd29yZA=="
    assert json_data["args"] == {"x": "aaa", "y": "bbb"}
    assert json_data["data"] == "test content"


@retry()
def test_client_post_data():
    client = Client(verify=False)
    auth_bearer = "bearerXXXXXXXXXXXXXXXXXXXX"
    headers = {"X-Test": "test"}
    params = {"x": "aaa", "y": "bbb"}
    data = {"key1": "value1", "key2": "value2"}
    response = client.post(
        "https://httpbin.org/anything",
        auth_bearer=auth_bearer,
        headers=headers,
        params=params,
        data=data,
    )
    assert response.status_code == 200
    json_data = response.json()
    assert json_data["headers"]["X-Test"] == "test"
    assert json_data["headers"]["Authorization"] == "Bearer bearerXXXXXXXXXXXXXXXXXXXX"
    assert json_data["args"] == {"x": "aaa", "y": "bbb"}
    received_data_dict = parse_qs(json_data["data"])
    assert received_data_dict == {"key1": ["value1"], "key2": ["value2"]}


@retry()
def test_client_impersonate():
    client = Client(impersonate="chrome_123", verify=False)
    response = client.get("https://tls.peet.ws/api/all")
    assert response.status_code == 200
    json_data = response.json()
    assert json_data["http_version"] == "h2"
    assert json_data["tls"]["ja4"].startswith("t13d")
    assert (
        json_data["http2"]["akamai_fingerprint_hash"]
        == "90224459f8bf70b7d0a8797eb916dbc9"
    )
